package com.cusc.nirvana.user.rnr.enterprise.config;

import com.alibaba.fastjson.JSON;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.common.DesensitizationFieId;
import com.cusc.nirvana.user.rnr.enterprise.common.EnterpriseIgnoreBase64ImgJsonSerializeFilter;
import com.cusc.nirvana.user.rnr.enterprise.util.DesensitizationUtil;
import com.cusc.nirvana.user.rnr.fp.client.FpUserOperationLogClient;
import com.cusc.nirvana.user.rnr.fp.dto.FpUserOperationLogDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

/**
 * Description: mvc日志输出
 * <br />
 * CreateDate 2021-11-12 10:26
 *
 * @author yuyi
 **/
@Aspect
@Component
public class WebLogAspect {

	private final static Logger LOGGER = LoggerFactory.getLogger(WebLogAspect.class);

	@Resource
	private FpUserOperationLogClient logClient;

	private static final String IP_UNKOWN="unknown";

	@Resource
	private ThreadPoolExecutor logThreadPool;

	/**
	 * 以 controller 包下定义的所有请求为切入点
	 */
	@Pointcut("execution(* com.cusc.nirvana.user.rnr.enterprise.controller.*.*(..))")
	public void webLog() {
	}

	/**
	 * 环绕
	 *
	 * @param proceedingJoinPoint
	 * @return
	 * @throws Throwable
	 */
	@Around("webLog()")
	public Object doAround(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
		long startTime = System.currentTimeMillis();
		String requestId = CuscStringUtils.generateUuid();

		ServletRequestAttributes sra = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		HttpServletRequest request = sra.getRequest();

		String url = request.getRequestURL().toString();
		String method = request.getMethod();

		String params =  "";
		//获取请求参数集合并进行遍历拼接
		if (HttpMethod.POST.name().equals(method)) {
			Object[] args = proceedingJoinPoint.getArgs();
			List<Object> filterArgsList = Arrays.stream(args).filter(arg -> !(arg instanceof ServletResponse)).collect(Collectors.toList());
			params = JSON.toJSONString(filterArgsList, new EnterpriseIgnoreBase64ImgJsonSerializeFilter());
		} else if (HttpMethod.GET.name().equals(method)) {
			params = request.getQueryString();
		}

		LOGGER.info("requestId:{} , request url:{} , method:{} , params:{}", requestId, url, method, params);
		// result的值就是被拦截方法的返回值
		Object result = null;
		try{
			result = proceedingJoinPoint.proceed();
			LOGGER.info("requestId:{} , response url:{} , result:{} , cost:{} ms", requestId, url,
					JSON.toJSONString(result, new EnterpriseIgnoreBase64ImgJsonSerializeFilter()),
					System.currentTimeMillis() - startTime);
		}catch (Exception e) {
			LOGGER.error("requestId:{} , response url:{} , e:{}, cost:{} ms", requestId, url,
					e, System.currentTimeMillis() - startTime);
			if (e instanceof CuscUserException) {
				CuscUserException cuscUserException = (CuscUserException) e;
				result = Response.createError(e.getMessage(),StringUtils.isBlank(cuscUserException.getCode()) ? "-1":cuscUserException.getCode());
			}
			throw e;
		} finally {
			MethodSignature methodSignature = (MethodSignature)proceedingJoinPoint.getSignature();
			Method method1 = methodSignature.getMethod();
			SysLog sysLog = method1.getAnnotation(SysLog.class);
			if (null != sysLog) {
				//记录日志
				saveLog(sysLog,request,params,result,proceedingJoinPoint,
						UserSubjectUtil.getUserId(),UserSubjectUtil.getTenantNo(),UserSubjectUtil.getOrganId());
			}
		}
		return result;
	}

	private void saveLog(SysLog sysLog,HttpServletRequest request,String params,Object result,
						 ProceedingJoinPoint proceedingJoinPoint,String userId,String tenantNo,String orgId) {
		logThreadPool.submit(() -> {
			try {
				FpUserOperationLogDTO logDto = new FpUserOperationLogDTO();
				logDto.setUuid(CuscStringUtils.generateUuid());
				logDto.setOptModule(sysLog.sysModule().getSysModule());
				logDto.setOptAction(sysLog.sysModule().getSysAction());
				logDto.setRequestIp(getClientIp(request));
				logDto.setRequestParameter(params);
				logDto.setRequestUrl(request.getRequestURL().toString());
				logDto.setCreator(userId);
				logDto.setTenantNo(tenantNo);
				logDto.setOrgId(orgId);
				if (result instanceof Response){
					Response response = (Response) result;
					logDto.setResponseCode(String.valueOf(response.getCode()));
					logDto.setResponseMsg(response.getMsg());
				}

				String optContent = sysLog.sysModule().getOptContent();
				if (StringUtils.isNotBlank(optContent)){
					List<String> desensitizeContent = new ArrayList<>();
					Object[] args = proceedingJoinPoint.getArgs();
					for (Object arg:args){
						Field[] fields = arg.getClass().getDeclaredFields();
						for (Field field:fields){
							DesensitizationFieId annotation = field.getDeclaredAnnotation(DesensitizationFieId.class);
							if (null != annotation) {
								try {
									field.setAccessible(true);
									Object o = field.get(arg);
									String content = o instanceof String ? (String)o:JSON.toJSONString(o);
									desensitizeContent.add(annotation.sort(),DesensitizationUtil.desensitizeByRule(annotation.desensitizationRule(),content));
								} catch (Exception e) {
									LOGGER.error("获取属性值失败");
								}
							}
						}
					}
					logDto.setOptContent(String.format(optContent,desensitizeContent.toArray()));
				}

				logClient.add(logDto);
			} catch (Exception e) {
				LOGGER.error("saveLog has error",e);
			}
		});
	}


	private static String getClientIp(HttpServletRequest request) {
		String ip = request.getHeader("x-forwarded-for");
		if (StringUtils.isBlank(ip) || "".equals(ip.trim()) || IP_UNKOWN.equalsIgnoreCase(ip)) {
			ip = request.getHeader("Proxy-Client-IP");
		}
		if (StringUtils.isBlank(ip) || "".equals(ip.trim()) || IP_UNKOWN.equalsIgnoreCase(ip)) {
			ip = request.getHeader("WL-Proxy-Client-IP");
		}
		if (StringUtils.isBlank(ip) || "".equals(ip.trim()) || IP_UNKOWN.equalsIgnoreCase(ip)) {
			ip = request.getRemoteAddr();
		}
		if (StringUtils.isBlank(ip)) {
			return "";
		}
		// 多个路由时，取第一个非unknown的ip
		final String[] arr = ip.split(",");
		for (final String str : arr) {
			if (!IP_UNKOWN.equalsIgnoreCase(str)) {
				ip = str;
				break;
			}
		}
		return ip;
	}

}
