package com.cusc.nirvana.user.rnr.enterprise.controller;

import com.cusc.nirvana.common.loader.StringUtils;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.eiam.constants.CommonYesOrNoEnum;
import com.cusc.nirvana.user.rnr.enterprise.common.SysModuleEnum;
import com.cusc.nirvana.user.rnr.enterprise.config.SysLog;
import com.cusc.nirvana.user.rnr.enterprise.dto.CompanyRnrResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.EMVinCardResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.EnterpriseRnrCertificationInfoDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.EnterpriseRnrFollowUpDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.EnterpriseRnrVinCardInfoDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.FileDownloadDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.VehicleEnterpriseRnrEchoDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IEnterpriseRnrService;
import com.cusc.nirvana.user.rnr.enterprise.service.IManufacturerRnrService;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.web.exception.AppGlobalException;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.*;
import java.net.*;
import java.util.Base64;


/**
 * 车企实名认证控制层
 *
 * @author huzl
 * @date 2022-04-19
 */
@RestController
@RequestMapping("/manufacturerRnr")
public class ManufacturerRnrController {

    @Value("${fileTemplate.manufacturer.templateId:template.xlsx}")
    private String manufacturerRnrTemplateFileId;


    @Resource
	private FileSystemClient fileSystemClient;

    @Autowired
    private IManufacturerRnrService manufacturerRnrService;

    @Autowired
    private IEnterpriseRnrService enterpriseRnrService;


    /**
     * 下载车企实名模板
     */
    @ApiOperation(value = "下载车企实名模板", notes = "下载车企实名模板")
    @GetMapping("/downloadTemplate")
    public Response downloadTemplate(HttpServletResponse response) {
        if (StringUtils.isBlank(manufacturerRnrTemplateFileId)) {
            return Response.createError("", "车企实名模板未配置");
        }

        FileDownloadDTO fileDownloadDTO = new FileDownloadDTO();
        fileDownloadDTO.setUuid(manufacturerRnrTemplateFileId);



		try {
			String base64 = fileSystemClient.getBase64(fileDownloadDTO).getData().getBase64();
			byte[] decode = Base64.getDecoder().decode(base64);
			// 以文件形式下载
			response.setHeader("Content-Disposition",
					"attachment;filename=" + URLEncoder.encode("template.xls", "utf-8"));
			response.getOutputStream().write(decode);

			response.getOutputStream().flush();
			response.getOutputStream().close();
		} catch (Exception e){
			throw new AppGlobalException("下载失败", e);
		}
        return Response.createSuccess();
    }


    /**
     * 车企实名认证上传Excel文件，需要传入文件系统id，从文件系统下载后解析，成功则继续，失败则删除原文件，新文件添加失败原因后重新上传至文件系统。
     *
     * @param dto
     * @return 单条数据
     */
    @ApiOperation(value = "校验车卡信息", notes = "企业实名认证校验车卡信息")
    @PostMapping("/verifyVinCards")
    public Response<EMVinCardResponseDTO> verifyVinCards(@Valid @RequestBody EnterpriseRnrVinCardInfoDTO dto) {
        return enterpriseRnrService.verifyVinCards(dto);
    }

    /**
     * 提交车企实名认证
     *
     * @param dto
     * @return 单条数据
     */
    @ApiOperation(value = "提交车企实名认证", notes = "提交车企实名认证")
    @PostMapping("/submitRnr")
	@SysLog(sysModule = SysModuleEnum.MANUFACTURER_WEB_RNR_FIRST)
    public Response<CompanyRnrResponseDTO> submitRnr(@Valid @RequestBody EnterpriseRnrCertificationInfoDTO dto) {
        dto.setIsVehicleCompany(CommonYesOrNoEnum.YES.getCode());
        return enterpriseRnrService.submitRnr(dto);
    }

    /**
     * 提交车企实名认证
     *
     * @return 单条数据
     */
    @ApiOperation(value = "提交车企实名认证-后续", notes = "提交车企实名认证-后续")
    @PostMapping("/submitRnrFollowUp")
	@SysLog(sysModule = SysModuleEnum.MANUFACTURER_WEB_RNR_FOLLOW_UP)
    public Response<CompanyRnrResponseDTO> submitRnrFollowUp(@Valid @RequestBody EnterpriseRnrFollowUpDTO bean) {
        return manufacturerRnrService.submitRnrFollowUp(bean);
    }

    /**
     * 查询车企详情
     *
     * @return 单条数据
     */
    @ApiOperation(value = "查询车企详情", notes = "查询车企详情")
    @PostMapping("/getDetail")
    public Response<VehicleEnterpriseRnrEchoDTO> getDetail() {
        //todo 获取用户id
        String userId = UserSubjectUtil.getUserId();
        String tenantNo = UserSubjectUtil.getTenantNo();
        //通过用户id查询当前组织
        String carOrganId = "";
        //判断当前组织是否是车企，如果不是则需要通过组织id找到上级车企信息
        return Response.createSuccess(manufacturerRnrService.findDetail(tenantNo, carOrganId));
    }
}
