package com.cusc.nirvana.user.rnr.enterprise.excel;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.exception.ExcelAnalysisStopException;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.constants.FuelTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.constants.PlaceOfOriginOfVehicleEnum;
import com.cusc.nirvana.user.rnr.enterprise.constants.VehicleChannelTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.constants.VehicleTypeEnum;
import com.cusc.nirvana.user.rnr.enterprise.dto.ImportSimDTO;
import com.cusc.nirvana.user.rnr.enterprise.excel.utils.EasyExcelUtil;
import com.cusc.nirvana.user.rnr.fp.client.FpCarInfoClient;
import com.cusc.nirvana.user.rnr.fp.dto.CarInfoBindingDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FpCarInfoDTO;
import com.cusc.nirvana.user.rnr.fp.dto.SimVehicleBindingResult;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @className: CarInfoListener
 * @description: 业务处理
 * @author: jk
 * @date: 2022/6/29 10:42
 * @version: 1.0
 **/
public class CarInfoListener extends AnalysisEventListener<CarInfoRow> {

    private final static Logger LOGGER = LoggerFactory.getLogger(VehicleSimListener.class);

    /**每隔500条提交数据处理*/
    private static final int BATCH_COUNT = 500;

    private FpCarInfoClient fpCarInfoClient;

    private SimFileHistoryDTO simFileHistoryDTO;

    private String errorExcelFile;

    private EasyExcelUtil easyExcelUtil;

    private ImportSimDTO importSimDTO;

    private String creator;

    private String tenantNo;

    /**存放批数据*/
    private Map<String, CarInfoEscalationRow> batchCache = new HashMap<>();

    public CarInfoListener(FpCarInfoClient fpCarInfoClient,
                              SimFileHistoryDTO simFileHistoryDTO, String errorExcelFile, ImportSimDTO importSimDTO,String creator,String tenantNo) {
        this.fpCarInfoClient = fpCarInfoClient;
        this.simFileHistoryDTO = simFileHistoryDTO;
        this.errorExcelFile = errorExcelFile;
        this.importSimDTO = importSimDTO;
        this.creator = creator;
        this.tenantNo = tenantNo;
    }


    @Override
    public void invokeHeadMap(Map<Integer, String> headMap, AnalysisContext context) {
        super.invokeHeadMap(headMap, context);
        if (headMap.size() < 20) {
            simFileHistoryDTO.setErrorInfo("文件标题不符合模板要求，文件标题：[" + headMap.values() + "]");
            throw new ExcelAnalysisStopException("文件不符合模板要求。文件标题：[" + headMap.values() + "]");
        }
    }

    @Override
    public void invoke(CarInfoRow carInfoRow, AnalysisContext analysisContext) {

        String failReason = validata(carInfoRow);

        CarInfoEscalationRow infoRow = rowToInfoRow(carInfoRow);
        if (CuscStringUtils.isNotEmpty(failReason)) {
            infoRow.setErrorInfo(failReason);
            writeErrorExcel(infoRow);
            return;
        }
        batchCache.put(infoRow.getVin(), infoRow);
        if (batchCache.size() >= BATCH_COUNT) {
            processData();
            batchCache.clear();
        }
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        processData();
        excelFinish();
    }

    @Override
    public void onException(Exception exception, AnalysisContext context) throws Exception {
        exception.printStackTrace();
        simFileHistoryDTO.setErrorInfo("车辆信息有问题，请下载文件查看详情");
        excelFinish();
        throw new ExcelAnalysisStopException("车辆信息有问题，请下载文件查看详情");
    }

    private void processData() {
        CarInfoBindingDTO bindingDTO = new CarInfoBindingDTO();
        bindingDTO.setOrganizationId(simFileHistoryDTO.getOrgUuid());
        bindingDTO.setTagUuid(importSimDTO.getTagUuid());
        bindingDTO.setCreator(simFileHistoryDTO.getCreator());
        List<FpCarInfoDTO> fpCarInfoDTOS = new ArrayList<>();
        String tagId =CuscStringUtils.generateUuid();
        batchCache.values().forEach(infoRow -> {
            FpCarInfoDTO dto = infoRowToDto(infoRow);
            dto.setUuid(CuscStringUtils.generateUuid());
            dto.setOrgid(importSimDTO.getOrgUuid());
            dto.setCreator(creator);
            dto.setTenantno(tenantNo);
            dto.setOperator(creator);
            dto.setTagId(tagId);
            fpCarInfoDTOS.add(dto);
        });

        bindingDTO.setBindingList(fpCarInfoDTOS);

        //入库处理
        Response<SimVehicleBindingResult> response = fpCarInfoClient.addBatch(bindingDTO);

        if (!response.isSuccess()) {
            fpCarInfoDTOS.forEach(simVehicleDTO -> {
                CarInfoEscalationRow infoRow = dtoToInfoRow(simVehicleDTO);
                infoRow.setErrorInfo("导入失败");
                writeErrorExcel(infoRow);
            });
            return;
        }
    }

    private CarInfoEscalationRow dtoToInfoRow(FpCarInfoDTO dto) {
        CarInfoEscalationRow infoRow = new CarInfoEscalationRow();
        CarInfoEscalationRow tmp = batchCache.get(dto.getVin());
        BeanUtils.copyProperties(tmp,infoRow);
        infoRow.setErrorInfo(dto.getFailReason());
        return infoRow;
    }

    private FpCarInfoDTO infoRowToDto(CarInfoEscalationRow infoRow) {
        FpCarInfoDTO dto = new FpCarInfoDTO();
        BeanUtils.copyProperties(infoRow,dto);
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        if(infoRow.getVehicleDepartureTime()!=null){
            dto.setVehicleDepartureTime(simpleDateFormat.format(infoRow.getVehicleDepartureTime()));
        }
        if(infoRow.getVehicleSalesTime()!=null) {
            dto.setVehicleSalesTime(simpleDateFormat.format(infoRow.getVehicleSalesTime()));
        }
        if(infoRow.getVehicleSalesUpdateTime()!=null) {
            dto.setVehicleSalesUpdateTime(simpleDateFormat.format(infoRow.getVehicleSalesUpdateTime()));
        }
        return dto;
    }

    /**
     * 生成错误文件信息
     * @param infoRow
     */
    private void writeErrorExcel(CarInfoEscalationRow infoRow) {
        simFileHistoryDTO.setErrorCount(simFileHistoryDTO.getErrorCount() + 1);
        if (easyExcelUtil == null) {
            easyExcelUtil = new EasyExcelUtil();
            easyExcelUtil.init(errorExcelFile, "Sheet1", CarInfoEscalationRow.class);
        }
        List<CarInfoEscalationRow> infoRows = new ArrayList<>();
        infoRows.add(infoRow);
        easyExcelUtil.addExcel(infoRows);
    }

    private void excelFinish() {
        if (easyExcelUtil != null) {
            easyExcelUtil.finish();
        }
    }

    private CarInfoEscalationRow rowToInfoRow(CarInfoRow carInfoRow) {
        CarInfoEscalationRow infoRow = new CarInfoEscalationRow();
        infoRow.setNameVehicleEnterprise(StringUtils.isEmpty(carInfoRow.getNameVehicleEnterprise())?"":carInfoRow.getNameVehicleEnterprise().trim());
        infoRow.setVin(StringUtils.isEmpty(carInfoRow.getVin())?"":carInfoRow.getVin().trim());
        infoRow.setVehicleType(StringUtils.isEmpty(carInfoRow.getVehicleType())?"null":carInfoRow.getVehicleType().trim());
        infoRow.setPlaceOfOriginOfVehicle(StringUtils.isEmpty(carInfoRow.getPlaceOfOriginOfVehicle())?"null":carInfoRow.getPlaceOfOriginOfVehicle().trim());
        infoRow.setVehicleDepartment(StringUtils.isEmpty(carInfoRow.getVehicleDepartment())?"":carInfoRow.getVehicleDepartment().trim());
        infoRow.setVehicleName(StringUtils.isEmpty(carInfoRow.getVehicleName())?"":carInfoRow.getVehicleName().trim());
        infoRow.setVehicleNum(StringUtils.isEmpty(carInfoRow.getVehicleNum())?"":carInfoRow.getVehicleNum().trim());
        infoRow.setVehicleModel(StringUtils.isEmpty(carInfoRow.getVehicleModel())?"":carInfoRow.getVehicleModel().trim());
        infoRow.setBodyColor(StringUtils.isEmpty(carInfoRow.getBodyColor())?"":carInfoRow.getBodyColor().trim());
        infoRow.setFuelType(StringUtils.isEmpty(carInfoRow.getFuelType())?null:carInfoRow.getFuelType().trim());
        infoRow.setEngineNum(StringUtils.isEmpty(carInfoRow.getEngineNum())?"":carInfoRow.getEngineNum().trim());
        infoRow.setMotorNum(StringUtils.isEmpty(carInfoRow.getMotorNum())?"":carInfoRow.getMotorNum().trim());
        infoRow.setLicensePlateNumber(StringUtils.isEmpty(carInfoRow.getLicensePlateNumber())?"":carInfoRow.getLicensePlateNumber().trim());
        infoRow.setVehicleChannelName(StringUtils.isEmpty(carInfoRow.getVehicleChannelName())?"":carInfoRow.getVehicleChannelName().trim());
        infoRow.setVehicleChannelType(StringUtils.isEmpty(carInfoRow.getVehicleChannelType())?null:carInfoRow.getVehicleChannelType().trim());
        infoRow.setVehicleStaffName(StringUtils.isEmpty(carInfoRow.getVehicleStaffName())?"":carInfoRow.getVehicleStaffName().trim());
        infoRow.setVehicleLogginAddress(StringUtils.isEmpty(carInfoRow.getVehicleLogginAddress())?"":carInfoRow.getVehicleLogginAddress().trim());
        infoRow.setVehicleDepartureTime(carInfoRow.getVehicleDepartureTime());
        infoRow.setVehicleSalesTime(carInfoRow.getVehicleSalesTime());
        infoRow.setVehicleSalesUpdateTime(carInfoRow.getVehicleSalesUpdateTime());
//        BeanUtils.copyProperties(carInfoRow,infoRow);

        return infoRow;
    }

    private String validata(CarInfoRow row) {
        String failReason = "";

        if (StringUtils.isBlank(row.getNameVehicleEnterprise())) {
            failReason = "道路机动车辆生产企业名称为空";
        }else {
            if(row.getNameVehicleEnterprise().length()>32){
                failReason = "道路机动车辆生产企业名称超长";
            }
        }

        if (StringUtils.isBlank(row.getVin())) {
            failReason = "VIN为空";
        }else {
            if(row.getVin().length()>64){
                failReason = "VIN超长";
            }
        }

        if (StringUtils.isBlank(row.getVehicleType())) {
            failReason = "车辆类型为空";
        }else {
            if(null == VehicleTypeEnum.getEnumByCode(row.getVehicleType())){
                failReason = "车辆类型与法规不一致";
            }
            if(row.getVehicleType().length()>2){
                failReason = "车辆类型超长";
            }
        }

        if (StringUtils.isBlank(row.getPlaceOfOriginOfVehicle())) {
            failReason = "车辆产地为空";
        }else {
            if(null == PlaceOfOriginOfVehicleEnum.getEnumByCode(row.getPlaceOfOriginOfVehicle())){
                failReason = "车辆产地与法规不一致";
            }

            if(row.getPlaceOfOriginOfVehicle().length()>2){
                failReason = "车辆产地超长";
            }
        }

        if (StringUtils.isBlank(row.getVehicleDepartment())) {
            failReason = "车辆品牌为空";
        }else {
            if(row.getVehicleDepartment().length()>32){
                failReason = "车辆品牌超长";
            }
        }

        if (StringUtils.isBlank(row.getVehicleName())) {
            failReason = "车辆名称为空";
        }else {
            if(row.getVehicleName().length()>32){
                failReason = "车辆名称超长";
            }
        }

        if (StringUtils.isBlank(row.getVehicleNum())) {
            failReason = "车辆型号为空";
        }else {
            if(row.getVehicleNum().length()>32){
                failReason = "车辆型号超长";
            }
        }

        if (StringUtils.isBlank(row.getVehicleModel())) {
            failReason = "车型通用名称为空";
        }else {
            if(row.getVehicleModel().length()>32){
                failReason = "车型通用名称超长";
            }
        }

        if (StringUtils.isBlank(row.getBodyColor())) {
            failReason = "车身颜色为空";
        }else {
            if(row.getBodyColor().length()>32){
                failReason = "车身颜色超长";
            }
        }

        if (StringUtils.isBlank(row.getFuelType())) {
            failReason = "燃料种类为空";
        }else {
            String[] fuelType = row.getFuelType().split(",");
            for(int i = 0 ;i<fuelType.length;i++){
                if(null == FuelTypeEnum.getEnumByCode(fuelType[i])){
                    failReason = "燃料种类与法规不一致";
                }
            }
            if(row.getFuelType().length()>32){
                failReason = "燃料种类超长";
            }
        }

//        if (StringUtils.isBlank(row.getEngineNum())) {
//            failReason = "发动机号码为空";
//        }
//
//        if (StringUtils.isBlank(row.getMotorNum())) {
//            failReason = "电动机序列号为空";
//        }

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String vehicleDepartureTime = simpleDateFormat.format(row.getVehicleDepartureTime());
        if (StringUtils.isBlank(vehicleDepartureTime)) {
            failReason = "车辆出厂时间为空";
        }
        String vehicleSalesTime=simpleDateFormat.format(row.getVehicleSalesTime());;
        if (StringUtils.isBlank(vehicleSalesTime)) {
            failReason = "车辆销售时间为空";
        }

//        if (StringUtils.isBlank(row.getVehicleSalesUpdateTime())) {
//            failReason = "车辆销售信息变更时间为空";
//        }
//
//        if (StringUtils.isBlank(row.getLicensePlateNumber())) {
//            failReason = "车牌号码为空";
//        }else {
//            if(row.getLicensePlateNumber().length()>64){
//                failReason = "车牌号码超长";
//            }
//        }

        if (StringUtils.isBlank(row.getVehicleChannelName())) {
            failReason = "车辆销售渠道名称为空";
        }else {
            if(row.getVehicleChannelName().length()>32){
                failReason = "车辆销售渠道名称超长";
            }
        }

        if (StringUtils.isBlank(row.getVehicleChannelType())) {
            failReason = "车辆销售渠道类型为空";
        }else {
            if(null == VehicleChannelTypeEnum.getEnumByCode(row.getVehicleChannelType())){
                failReason = "车辆销售渠道类型不符合法规要求";
            }
            if(row.getVehicleChannelType().length()>2){
                failReason = "车辆销售渠道类型超长";
            }
        }

        if (StringUtils.isBlank(row.getVehicleStaffName())) {
            failReason = "车辆销售渠道员工姓名为空";
        }else {
            if(row.getVehicleStaffName().length()>32){
                failReason = "车辆销售渠道员工姓名超长";
            }
        }

        if (StringUtils.isBlank(row.getVehicleLogginAddress())) {
            failReason = "车辆销售渠道地址为空";
        }else {
            if(row.getVehicleLogginAddress().length()>32){
                failReason = "车辆销售渠道地址超长";
            }
        }
        return failReason;
    }
}
