package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cache.CacheFactory;
import com.cache.local.guava.page.Page;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.eiam.client.OrganizationClient;
import com.cusc.nirvana.user.eiam.client.UserClient;
import com.cusc.nirvana.user.eiam.client.UserRoleClient;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.eiam.dto.UserDTO;
import com.cusc.nirvana.user.eiam.dto.UserRoleDTO;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.common.RedisConstants;
import com.cusc.nirvana.user.rnr.enterprise.constants.OrdinaryAdminEnum;
import com.cusc.nirvana.user.rnr.enterprise.constants.ResponseCode;
import com.cusc.nirvana.user.rnr.enterprise.dto.DistributorAddDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.DistributorChangeAdminDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.DistributorDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.DistributorPageQueryDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IDistributorService;
import com.cusc.nirvana.user.rnr.enterprise.service.IUserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author stayAnd
 * @date 2022/4/14
 */
@Service
@Slf4j
public class DistributorServiceImpl implements IDistributorService {

	@Resource
	private IUserService userService;

	@Resource
	private OrganizationClient organizationClient;

	@Resource
	private UserClient userClient;

	@Resource
	private CacheFactory cacheFactory;

	@Resource
	private UserRoleClient userRoleClient;

	/**
	 * 默认经销商管理员角色code
	 */
	@Value("${distributor.user.role.id:201}")
	private String distributorRoleCode;

	@Value("${user.eiam.applicationId:4}")
	private String applicationId;


	@Override
	public void add(DistributorAddDTO dto) {
		UserDTO queryUserDTO = userService.queryUserInInfoByPhone(dto.getPhone());
		if (null != queryUserDTO) {
			throw new CuscUserException(ResponseCode.USER_PHONE_EXISTS.getCode(),ResponseCode.USER_PHONE_EXISTS.getMsg());
		}

		//获取当前的用户
		String userId = UserSubjectUtil.getUserId();
		String organId = userService.getOrganId(userId,UserSubjectUtil.getTenantNo());
		if (StringUtils.isBlank(organId)) {
			throw new CuscUserException(ResponseCode.USER_ORGAN_NOT_FOUND.getCode(),ResponseCode.USER_ORGAN_NOT_FOUND.getMsg());
		}

		UserDTO userDto = dto.toUserDto();
		userDto.setTenantNo(UserSubjectUtil.getTenantNo());
		userDto.setApplicationId(applicationId);
		userDto.setOrdinaryAdmin(OrdinaryAdminEnum.DISTRIBUTOR.getCode());
		UserDTO addUserDto = userService.addUser(userDto);

		//添加经销商(组织)
		OrganizationDTO organDto = dto.toOrganDTO();
		//设置经销商的上级节点为当前车企
		organDto.setParentId(organId);
		organDto.setTenantNo(UserSubjectUtil.getTenantNo());
		organDto.setOrganCode(generateOrganCode());
		Response<OrganizationDTO> organizationResponse = organizationClient.add(organDto);
		OrganizationDTO organizationDTO = organizationResponse.getData();

		//新增经销商管理员及其关系
		userService.addUserRelationRole(addUserDto.getUuid(),organizationDTO.getUuid(),distributorRoleCode);
	}

	@Override
	public PageResult<DistributorDTO> page(DistributorPageQueryDTO dto) {
		//获取当前的用户
		String userId = UserSubjectUtil.getUserId();
		//当前用户的组织id
		String organId = userService.getOrganId(userId,UserSubjectUtil.getTenantNo());

		OrganizationDTO queryDto = dto.toPageQueryDto();
		queryDto.setParentId(organId);
		Response<PageResult<OrganizationDTO>> organResponse = organizationClient.queryByPage(queryDto);

		PageResult<OrganizationDTO> page = organResponse.getData();

		List<DistributorDTO> dtoList = page.getList().stream().map(organizationDTO -> {
			DistributorDTO distributorDTO = new DistributorDTO();
			distributorDTO.setId(organizationDTO.getUuid());
			distributorDTO.setDistributorNo(organizationDTO.getOrganCode());
			distributorDTO.setDistributorName(organizationDTO.getOrganName());
			distributorDTO.setDistributorAdminUserName(organizationDTO.getAdminName());
			distributorDTO.setDistributorAdminUserAccount(organizationDTO.getAdminAccount());
			distributorDTO.setDistributorAdminUserId(organizationDTO.getAdminUserId());
			distributorDTO.setPhone(organizationDTO.getAdminPhone());
			distributorDTO.setStatus(organizationDTO.getStatus());
			distributorDTO.setRemark(organizationDTO.getComment());
			distributorDTO.setCreateTime(organizationDTO.getCreateTime());
			return distributorDTO;
		}).collect(Collectors.toList());

		return new PageResult<>(dtoList,page.getTotalCount(),page.getPageSize(),page.getCurrPage());
	}


	@Override
	public Map<String,Object> pageToMap(DistributorPageQueryDTO dto) {
		Map<String,Object> returnMap = new HashMap<>();
		//获取当前的用户
		String userId = UserSubjectUtil.getUserId();
		//当前用户的组织id
		String organId = userService.getOrganId(userId,UserSubjectUtil.getTenantNo());
		if(StringUtils.isEmpty(organId)){
			returnMap.put("errorMsg","该用户没有组织id");
			returnMap.put("errorCode",10001);
			return returnMap;
		}

		OrganizationDTO queryDto = dto.toPageQueryDto();
		queryDto.setParentId(organId);
		Response<PageResult<OrganizationDTO>> organResponse = organizationClient.queryByPage(queryDto);

		PageResult<OrganizationDTO> page = organResponse.getData();

		List<DistributorDTO> dtoList = page.getList().stream().map(organizationDTO -> {
			DistributorDTO distributorDTO = new DistributorDTO();
			distributorDTO.setId(organizationDTO.getUuid());
			distributorDTO.setDistributorNo(organizationDTO.getOrganCode());
			distributorDTO.setDistributorName(organizationDTO.getOrganName());
			distributorDTO.setDistributorAdminUserName(organizationDTO.getAdminName());
			distributorDTO.setDistributorAdminUserAccount(organizationDTO.getAdminAccount());
			distributorDTO.setDistributorAdminUserId(organizationDTO.getAdminUserId());
			distributorDTO.setPhone(organizationDTO.getAdminPhone());
			distributorDTO.setStatus(organizationDTO.getStatus());
			distributorDTO.setRemark(organizationDTO.getComment());
			distributorDTO.setCreateTime(organizationDTO.getCreateTime());
			return distributorDTO;
		}).collect(Collectors.toList());

		PageResult<DistributorDTO> returnList = new PageResult<>(dtoList,page.getTotalCount(),page.getPageSize(),page.getCurrPage());
		if(returnList.getList().size()<=0){
			returnMap.put("errorMsg","该用户未查询相关经销商");
			returnMap.put("errorCode",10001);
			return returnMap;
		}else{
			returnMap.put("errorCode",10000);
			returnMap.put("returnList",returnList);
			return returnMap;
		}
	}

	@Override
	public void update(DistributorDTO distributorDTO) {
		OrganizationDTO updateDto = new OrganizationDTO();
		updateDto.setUuid(distributorDTO.getId());
		updateDto.setOrganName(distributorDTO.getDistributorName());
		updateDto.setComment(distributorDTO.getRemark());
		updateDto.setTenantNo(UserSubjectUtil.getTenantNo());
		organizationClient.update(updateDto);
	}


	@Override
	public void changeAdmin(DistributorChangeAdminDTO dto) {
		if (dto.getNewAdminUserId() == null) {
			UserDTO userDTO = userService.queryUserInInfoByPhone(dto.getCreateAdminPhone());
			if (null != userDTO) {
				throw new CuscUserException(ResponseCode.USER_PHONE_EXISTS.getCode(),ResponseCode.USER_PHONE_EXISTS.getMsg());
			}
		}

		if (null == dto.getNewAdminUserId()) {
			//创建用户
			UserDTO userAdminDto = new UserDTO();
			userAdminDto.setOrdinaryAdmin(1);
			userAdminDto.setUserName(dto.getCreateAdminAccount());
			userAdminDto.setFullName(dto.getCreateAdminUserName());
			userAdminDto.setPhone(dto.getCreateAdminPhone());
			userAdminDto.setPassword(dto.getCreateAdminPassword());
			userAdminDto.setTenantNo(UserSubjectUtil.getTenantNo());
			userAdminDto.setApplicationId(applicationId);
			UserDTO userDTO = userService.addUser(userAdminDto);
			userService.addUserRelationRole(userDTO.getUuid(),dto.getId(),distributorRoleCode);
		} else {
			//选中的员工 赋值新的角色管理员
			UserDTO newAdminDto = new UserDTO();
			newAdminDto.setUuid(dto.getNewAdminUserId());
			newAdminDto.setTenantNo(UserSubjectUtil.getTenantNo());
			newAdminDto.setApplicationId(applicationId);
			newAdminDto.setOrdinaryAdmin(1);
			userClient.update(newAdminDto);

			UserRoleDTO newUserRoleAdminDto = new UserRoleDTO();
			newUserRoleAdminDto.setUserId(dto.getNewAdminUserId());
			newUserRoleAdminDto.setRoleId(userService.getRoleId(distributorRoleCode));
			userRoleClient.updateByUserId(newUserRoleAdminDto);
		}

		//取消当前经销商管理员
		UserDTO cancelAdminDto = new UserDTO();
		cancelAdminDto.setUuid(dto.getCurrentAdminUserId());
		cancelAdminDto.setTenantNo(UserSubjectUtil.getTenantNo());
		cancelAdminDto.setApplicationId(applicationId);
		cancelAdminDto.setOrdinaryAdmin(0);
		userClient.update(cancelAdminDto);

		UserRoleDTO cancelRoleAdminDto = new UserRoleDTO();
		cancelRoleAdminDto.setTenantNo(UserSubjectUtil.getTenantNo());
		cancelRoleAdminDto.setApplicationId(applicationId);
		cancelRoleAdminDto.setUserId(dto.getCurrentAdminUserId());
		userRoleClient.delBatchUser(cancelRoleAdminDto);

	}


	public String generateOrganCode() {
		try {
			Calendar calendar = Calendar.getInstance();
			int year = calendar.get(Calendar.YEAR);
			Long aLong = cacheFactory.getStringService().setValueIncr(RedisConstants.ORG_ORGAN_CODE_PREFIX + year, 1);
			return String.valueOf(year * 10000 + aLong);
		} catch (Exception e) {
			log.error("generateOrganCode has error",e);
		}
		return "";
	}
}
