package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.rnr.enterprise.dto.VehicleCardRnrDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.ISmsService;
import com.cusc.nirvana.user.rnr.enterprise.service.ISurplusCardBindService;
import com.cusc.nirvana.user.rnr.enterprise.service.IVehicleCardService;
import com.cusc.nirvana.user.rnr.fp.client.CardUnBindClient;
import com.cusc.nirvana.user.rnr.fp.client.FpSurplusCardBindClient;
import com.cusc.nirvana.user.rnr.fp.client.VinCardClient;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import com.cusc.nirvana.user.rnr.mg.constants.*;
import com.cusc.nirvana.user.rnr.mg.dto.*;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author yubo
 * @since 2022-05-03 09:16
 */
@Service
@Slf4j
public class SurplusCardBindServiceImpl implements ISurplusCardBindService {

    @Autowired
    VinCardClient vinCardClient;

    @Autowired
    CardUnBindClient cardUnBindClient;

    @Autowired
    ISmsService smsService;

    @Autowired
    FpSurplusCardBindClient fpSurplusCardBindClient;

    @Autowired
    IVehicleCardService vehicleCardService;

    /**
     * Description: 提交绑定剩余卡
     * 1.发送短信
     * 2.等待短信回复
     * 3.将实名id与iccid绑定
     **/
    @Override
    public Response bindSurplusCard(VehicleCardRnrDTO bean) {
        //校验车卡关系
        Response response = validCard(bean);
        if (!response.isSuccess()) {
            return response;
        }

        //获取实名信息
        Response<MgRnrInfoDTO> rnrInfo = cardUnBindClient.getRnrInfo(new FpVehicleCardRnrDTO().setRnrId(bean.getRnrId()));
        if (!rnrInfo.isSuccess()) {
            return Response.createError(rnrInfo.getMsg(), rnrInfo.getCode());
        }
        MgRnrInfoDTO rnrInfoDTO = rnrInfo.getData();
        if (rnrInfoDTO == null) {
            return Response.createError("未获取到对应的实名信息", ResponseCode.INVALID_DATA.getCode());
        }

        RnrRelationDTO rnrRelationDTO = toRelationDto(bean, rnrInfoDTO);

        Response<String> integerResponse = fpSurplusCardBindClient.surplusCardBind(rnrRelationDTO);
        if (!integerResponse.isSuccess()) {
            return Response.createError(integerResponse.getMsg(), integerResponse.getCode());
        }

        return Response.createSuccess(new MgBizNoticeDTO(rnrRelationDTO.getOrder().getUuid()));
    }

    /**
     * Description: 查询绑定结果
     **/
    @Override
    public Response<SurplusQueryResponseDTO> queryResult(SurplusQueryRequestDTO bean) {
        return fpSurplusCardBindClient.queryResult(bean);
    }

    //----------------------------私有方法区------------

    //校验车卡关系
    private Response validCard(VehicleCardRnrDTO bean) {
        //车卡关系校验
        List<VinCardDTO> vinCardDTOS = bean.getIccidList().stream().map(iccid -> new VinCardDTO(bean.getVin(), iccid)).collect(Collectors.toList());
        Response<VerifyVinCardResponseDTO> response = vehicleCardService.verifyVinCardBatch(vinCardDTOS);
        if (!response.isSuccess()) {
            return Response.createError(response.getMsg(), response.getCode());
        }
        if (response.getData() == null || CollectionUtils.isEmpty(response.getData().getIccidVerifyResults())) {
            return Response.createError("ICCID不能为空", ResponseCode.INVALID_DATA.getCode());
        }
        //车卡是否已绑定
        Optional<VinCardVerityResult> first = response.getData().getIccidVerifyResults().stream().filter(r -> !r.isExist()).findFirst();
        if (first.isPresent()) {
            return Response.createError(first.get().getMsg(), ResponseCode.INVALID_DATA.getCode());
        }
        //根据vin查询所有已经实名的iccid
        Response<List<MgRnrCardInfoDTO>> cardListResponse = cardUnBindClient.getCardList(new FpVehicleCardRnrDTO().setVin(bean.getVin()));
        if (!cardListResponse.isSuccess()) {
            return Response.createError(cardListResponse.getMsg(), cardListResponse.getCode());
        }
        if (CollectionUtils.isEmpty(cardListResponse.getData())) {
            return Response.createError("VIN【" + bean.getVin() + "】没有已实名的卡", ResponseCode.INVALID_DATA.getCode());
        }
        Set<String> rnrCards = cardListResponse.getData().stream()
                .filter(card -> card.getRnrStatus() == RnrStatus.RNR.getCode())
                .map(c -> c.getIccid()).collect(Collectors.toSet());

        //卡是否已经实名了
        Optional<String> find = bean.getIccidList().stream().filter(iccid -> rnrCards.contains(iccid)).findFirst();
        if (find.isPresent()) {
            return Response.createError("卡【" + find.get() + "】已实名", ResponseCode.INVALID_DATA.getCode());
        }

        //实名业务id是否正确
        String rnrId = cardListResponse.getData().get(0).getRnrId();
        if (!StringUtils.equals(rnrId, bean.getRnrId())) {
            return Response.createError("实名业务和VIN不匹配", ResponseCode.INVALID_DATA.getCode());
        }

        return Response.createSuccess();
    }

    /**
     * 转成relation dto
     *
     * @param bean
     * @param rnrInfoDTO
     * @return
     */
    private RnrRelationDTO toRelationDto(VehicleCardRnrDTO bean, MgRnrInfoDTO rnrInfoDTO) {
        RnrRelationDTO relationDTO = new RnrRelationDTO();
        relationDTO.setInfo(rnrInfoDTO);
        relationDTO.setOrder(createOrder(rnrInfoDTO));
        relationDTO.setIsTrust(0);
        relationDTO.setIsSecondHandCar(0);
        relationDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        relationDTO.setCardList(createCardList(relationDTO.getOrder().getUuid(), bean, rnrInfoDTO));
        return relationDTO;
    }

    /**
     * 获取卡信息
     *
     * @param bean
     * @param info
     * @return
     */
    private List<MgRnrCardInfoDTO> createCardList(String orderId, VehicleCardRnrDTO bean, MgRnrInfoDTO info) {
        return bean.getIccidList().stream()
                .map(iccid -> {
                    MgRnrCardInfoDTO cardInfoDTO = new MgRnrCardInfoDTO();
                    cardInfoDTO.setUuid(CuscStringUtils.generateUuid());
                    cardInfoDTO.setIotId(bean.getVin());
                    cardInfoDTO.setIccid(iccid);
                    cardInfoDTO.setRnrId(info.getUuid());
                    cardInfoDTO.setOrderId(orderId);
                    cardInfoDTO.setTenantNo(UserSubjectUtil.getTenantNo());
                    cardInfoDTO.setCreator(UserSubjectUtil.getUserId());
                    cardInfoDTO.setOperator(UserSubjectUtil.getUserId());
                    cardInfoDTO.setSerial_number(info.getSerial_number());
                    cardInfoDTO.setNoticeStatus(NoticeStatusEnum.NONEED.getCode());
                    cardInfoDTO.setRnrStatus(RnrStatus.INIT.getCode());
                    cardInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Bind.getCode());
                    return cardInfoDTO;
                }).collect(Collectors.toList());
    }

    /**
     * 插入订单信息
     *
     * @param rnrInfoDTO
     * @return
     */
    private RnrOrderDTO createOrder(MgRnrInfoDTO rnrInfoDTO) {
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setOrderType(RnrOrderType.ONE_CAR_MORE_CARD_BIND.getCode());
        rnrOrderDTO.setRnrId(rnrInfoDTO.getUuid());
        rnrOrderDTO.setUuid(CuscStringUtils.generateUuid());
        rnrOrderDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        rnrOrderDTO.setOperator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setCreator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.PORTAL.getCode());
        rnrOrderDTO.setSerialNumber(rnrInfoDTO.getSerial_number());
        rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.TO_EXAMINE.getCode());
        rnrOrderDTO.setAutoRnr(true);
        rnrOrderDTO.setAuditType(RnrOrderAuditTypeEnum.AUTO.getCode());
        rnrOrderDTO.setSendWorkOrder(false);
        rnrOrderDTO.setIsBatchOrder(0);
        rnrOrderDTO.setRnrBizzType(RnrBizzTypeEnum.Bind.getCode());
		rnrOrderDTO.setOrgId(rnrInfoDTO.getOrgId());
        return rnrOrderDTO;
    }

}
