package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.enterprise.dto.EMVinCardResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.ExcelSheetDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.FileDownloadDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.UnBindIccidDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.VehicleCardRnrDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.VinDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IVehicleCardService;
import com.cusc.nirvana.user.rnr.enterprise.util.DesensitizationUtil;
import com.cusc.nirvana.user.rnr.enterprise.util.FileUtil;
import com.cusc.nirvana.user.rnr.fp.client.CardUnBindClient;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.client.NewVinCardClient;
import com.cusc.nirvana.user.rnr.fp.client.VinCardClient;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import com.cusc.nirvana.user.rnr.mg.constants.RnrStatus;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.*;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author yubo
 * @since 2022-04-18 09:26
 */
@Service
@Slf4j
public class VehicleCardServiceImpl implements IVehicleCardService {

	@Resource
	private FileSystemClient fileSystemClient;

    @Autowired
    VinCardClient vinCardClient;

    @Autowired
    NewVinCardClient newVinCardClient;

    @Autowired
    CardUnBindClient cardUnBindClient;

    @Value("${fileTemplate.manufacturer.fileSize: 500}")
    private String fileSize;



    /**
     * 车卡关系和卡校验
     *
     * @param vinCardDTO
     * @return
     */
    @Override
    public Response<VerifyVinCardResponseDTO> verifyVinCard(@Valid VinCardInfoDTO vinCardDTO) {
        //调用批量验证接口
        List<VinCardDTO> vinCardDTOS
                = vinCardDTO.getIccidList().stream().map(iccid -> new VinCardDTO(vinCardDTO.getVin(), iccid)).collect(Collectors.toList());

        return verifyVinCardBatch(vinCardDTOS);
    }

    /**
     * 批量车卡关系和卡校验
     *
     * @param verifyDTOs
     * @return
     */
    @Override
    public Response<VerifyVinCardResponseDTO> verifyVinCardBatch(List<VinCardDTO> verifyDTOs) {
        //进行vin和iccid的验证
        Response<VerifyVinCardResponseDTO> response = vinCardClient.verifyVinCard(verifyDTOs);
        if (!response.isSuccess()) {
            return response;
        }

        return Response.createSuccess(response.getData());
    }

    @Override
    public Response<IccidListDTO> getIccidListByVin(String vin) {
        Response<VinCardInfoDTO> iccidListByVin = vinCardClient.queryVinCard(vin);
        if (!iccidListByVin.getSuccess()) {
            return Response.createSuccess(false);
        }
        return Response.createSuccess(iccidListByVin.getData());
    }

    //解析excel
    @Override
    public Response<List<VinCardDTO>> checkIccidList(String fileId){
        //校验文件大小，调文件系统接口  /query/minfo/{uuid}  单位kb
        FileRecordDTO data = new FileRecordDTO();
        data.setUuid(fileId);
        Response<FileRecordDTO> fileRecordResponse = fileSystemClient.getFileSize(data);
        log.info("获取文件信息",fileRecordResponse.getData());
        if(fileRecordResponse.isSuccess()){
            data = fileRecordResponse.getData();
        }
        // 校验文件大小
        if(Double.parseDouble(fileSize) < data.getFileSize()){
            return Response.createError("文件大小限制为500k");
        }
        log.info("下载文件");
        Response<List<VinCardDTO>> listResponse = parseExcel(fileId);
        if(!listResponse.isSuccess() || listResponse.getData() == null || listResponse.getData().size()<1){
            return Response.createError("获取校验通过的iccids失败");
        }
        List<VinCardDTO> cardDTOLists  = listResponse.getData();
        log.info("vin和iccid的验证 : " + cardDTOLists.toString());
        Response<VerifyVinCardResponseDTO> verifyVinCardResponseDTOResponse = verifyVinCardBatch(cardDTOLists);
        if (!verifyVinCardResponseDTOResponse.isSuccess()) {
            return Response.createError(verifyVinCardResponseDTOResponse.getMsg(), verifyVinCardResponseDTOResponse.getCode());
        }
        int size = verifyVinCardResponseDTOResponse.getData().getIccidVerifyResults().stream().filter(s -> s.equals(false)).collect(Collectors.toList()).size();
        if(size>0){
            return Response.createError("车卡关系校验失败",cardDTOLists);
        }else {
            return Response.createSuccess("车卡关系校验成功",cardDTOLists);
        }

    }

    @Override
    public Response<UnBindIccidDTO> getUnBindIccidList(VinDTO vinDTO) {
        //根据vin查询所有iccid
        Response<VinCardInfoDTO> response = vinCardClient.queryVinCard(vinDTO.getVin());
        if (!response.isSuccess()) {
            return Response.createError(response.getMsg(), response.getCode());
        }
        if (response.getData() == null || CollectionUtils.isEmpty(response.getData().getIccidList())) {
            return Response.createError("VIN【" + vinDTO.getVin() + "】没有已绑定的卡", ResponseCode.INVALID_DATA.getCode());
        }

        //根据vin查询所有已经实名的iccid
        Response<List<MgRnrCardInfoDTO>> cardListResponse = cardUnBindClient.getCardList(new FpVehicleCardRnrDTO().setVin(vinDTO.getVin()));
        if (!cardListResponse.isSuccess()) {
            return Response.createError(cardListResponse.getMsg(), cardListResponse.getCode());
        }

        UnBindIccidDTO unBindIccidDTO = new UnBindIccidDTO();

        String rnrId;
        //过滤掉已经实名的iccid
        List<String> iccidList = response.getData().getIccidList();
        if (!CollectionUtils.isEmpty(cardListResponse.getData())) {
            Set<String> bindIccids = cardListResponse.getData().stream()
                    .filter(card -> card.getRnrStatus() == RnrStatus.RNR.getCode())
                    .map(card -> card.getIccid()).collect(Collectors.toSet());
            iccidList = iccidList.stream().filter(id -> !bindIccids.contains(id)).distinct().collect(Collectors.toList());
            rnrId = cardListResponse.getData().get(0).getRnrId();
        } else {
            return Response.createError("VIN【" + vinDTO.getVin() + "】没有已实名的卡", ResponseCode.INVALID_DATA.getCode());
        }
        if (CollectionUtils.isEmpty(iccidList)) {
            return Response.createError("VIN【" + vinDTO.getVin() + "】没有未实名的卡", ResponseCode.INVALID_DATA.getCode());
        }

        //获取实名信息
        Response<MgRnrInfoDTO> rnrInfo = cardUnBindClient.getRnrInfo(new FpVehicleCardRnrDTO().setRnrId(rnrId));
        if (!rnrInfo.isSuccess()) {
            return Response.createError(rnrInfo.getMsg(), rnrInfo.getCode());
        }
        MgRnrInfoDTO rnrInfoDTO = rnrInfo.getData();
        if (rnrInfoDTO == null) {
            return Response.createError("未获取到对应的实名信息", ResponseCode.INVALID_DATA.getCode());
        }

        unBindIccidDTO.setIccidList(iccidList);
        unBindIccidDTO.setVin(vinDTO.getVin());
        unBindIccidDTO.setGender(rnrInfoDTO.getGender());
        unBindIccidDTO.setRnrId(rnrInfoDTO.getUuid());
        unBindIccidDTO.setName(DesensitizationUtil.desensitizeName(rnrInfoDTO.getFullName()));
        unBindIccidDTO.setPhone(DesensitizationUtil.desensitizePhone(rnrInfoDTO.getPhone()));

        return Response.createSuccess(unBindIccidDTO);
    }

    @Override
    public Response<VehicleCardRnrDTO> getBindIccidList(VinDTO vinDTO) {
        //根据vin查询所有已经实名的iccid
        Response<List<MgRnrCardInfoDTO>> cardListResponse = cardUnBindClient.getCardList(new FpVehicleCardRnrDTO().setVin(vinDTO.getVin()));
        if (!cardListResponse.isSuccess()) {
            return Response.createError(cardListResponse.getMsg(), cardListResponse.getCode());
        }

        VehicleCardRnrDTO vehicleCardRnrDTO = new VehicleCardRnrDTO();
        if(!CollectionUtils.isEmpty(cardListResponse.getData())){
            List<String> iccids = cardListResponse.getData().stream()
                    .filter(card -> card.getRnrStatus() == RnrStatus.RNR.getCode())
                    .map(c -> c.getIccid()).collect(Collectors.toList());
            vehicleCardRnrDTO.setVin(vinDTO.getVin());
            vehicleCardRnrDTO.setIccidList(iccids);
            vehicleCardRnrDTO.setRnrId(cardListResponse.getData().get(0).getRnrId());
        }

        return Response.createSuccess(vehicleCardRnrDTO);
    }

    /**
     * 新车实名查询iccid，如果已经有实名信息，提醒走一车多卡
     * @param vin
     * @return
     */
    @Override
    public Response<IccidListDTO> getNewCarUnBindIccidList(String vin) {
        //根据vin查询所有iccid
        VinCardQueryDTO queryDTO = new VinCardQueryDTO();
        queryDTO.setVin(vin);
        Response<VinCardResultDTO> response = newVinCardClient.queryUnBindCardByVin(queryDTO);
        if (!response.isSuccess()) {
            return Response.createError(response.getMsg(), response.getCode());
        }
        if (response.getData() == null || CollectionUtils.isEmpty(response.getData().getIccidList())) {
            return Response.createError("VIN【" + vin + "】没有已绑定的卡", ResponseCode.INVALID_DATA.getCode());
        }

        //根据vin查询所有已经实名的iccid
        Response<List<MgRnrCardInfoDTO>> cardListResponse = cardUnBindClient.getCardList(new FpVehicleCardRnrDTO().setVin(vin));
        if (!cardListResponse.isSuccess()) {
            return Response.createError(cardListResponse.getMsg(), cardListResponse.getCode());
        }

        if(!CollectionUtils.isEmpty(cardListResponse.getData())){
            return Response.createError("VIN【" + vin + "】存在已实名的卡，请通过一车多卡进行绑定",ResponseCode.INVALID_DATA.getCode());
        }

        return Response.createSuccess(response.getData());

    }

    /**
     * 解析excel 获取校验成功的列表
     * @param fileId
     * @return
     */
    @Override
    public Response<List<VinCardDTO>> parseExcel(String fileId){
        List<VinCardDTO> cardDTOLists = new ArrayList<>();
        List<VinCardVerityResult> cardDTOList = new ArrayList<>();
        FileDownloadDTO fileDownloadDTO = new FileDownloadDTO(fileId, "");
        //InputStream download = fileService.download(fileDownloadDTO);
        try {
			String base64 = fileSystemClient.getBase64(fileDownloadDTO).getData().getBase64();
			ByteArrayInputStream inputStream = new ByteArrayInputStream(Base64.getDecoder().decode(base64));
			cardDTOList = FileUtil.convertExcelToVinCardLists(inputStream, fileId);
        } catch (Exception e) {
            log.error("读取车卡列表失败", e);
            //return Response.createError("读取车卡列表失败", ResponseCode.INVALID_DATA.getCode());
        }
        if (cardDTOList.size()>5000){
            Response.createError("文件条数应小于5000");
        }
        List<VinCardVerityResult> vinCardVerityResultList = cardDTOList.stream().filter(s->s.isExist()).collect(Collectors.toList());
        for (VinCardVerityResult result: vinCardVerityResultList){
            cardDTOLists.add(new VinCardDTO(result.getVin(),result.getIccid()));
        }
        return Response.createSuccess("获取校验正确iccid列表",cardDTOLists);
    }




    /**
     * 根据文件id下载并读取excel，进行vin和iccid校验，并上传处理失败的记录到文件系统
     *
     * @param fileId
     * @return
     */
    @Override
    public Response<EMVinCardResponseDTO> verifyAndUploadVinCard(String fileId) {

        //校验文件大小，调文件系统接口
        FileRecordDTO data = new FileRecordDTO();
        data.setUuid(fileId);
        Response<FileRecordDTO> fileRecordResponse = fileSystemClient.getFileSize(data);
        log.info("获取文件信息{}", fileRecordResponse.getData());
        if (fileRecordResponse.isSuccess()) {
            data = fileRecordResponse.getData();
        }
        // 校验文件大小
        if (Double.parseDouble(fileSize) < data.getFileSize()) {
            return Response.createError("文件大小限制为500k");
        }

        List<VinCardDTO> cardDTOList = null;
        try {
            //cardDTOList = fileService.downloadAndToVinCardList(fileId);
			com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO fileDownloadDTO = new FileDownloadDTO();
			fileDownloadDTO.setUuid(fileId);
			String base64 = fileSystemClient.getBase64(fileDownloadDTO).getData().getBase64();
			ByteArrayInputStream inputStream = new ByteArrayInputStream(Base64.getDecoder().decode(base64));
			cardDTOList = FileUtil.convertExcelToVinCardList(inputStream, fileId);
        } catch (Exception e) {
            log.error("读取车卡列表失败", e);
            return Response.createError("读取车卡列表失败", ResponseCode.INVALID_DATA.getCode());
        }

        Response<VerifyVinCardResponseDTO> response = verifyVinCardBatch(cardDTOList);
        log.info("verifyAndUploadVinCard response", response.toString());
        if (!response.isSuccess()) {
            return Response.createError(response.getMsg(), response.getCode());
        }

        //校验结果处理，计算失败次数
        List<VinCardVerityResult> iccidVerifyResults = response.getData().getIccidVerifyResults();

        //判断是否又重复的iccid
        List<String> repeatIccIdList = iccidVerifyResults.stream().filter(VinCardVerityResult::isExist)
                .collect(Collectors.groupingBy(VinCardVerityResult::getIccid))
                .entrySet().stream().filter(entry -> entry.getValue().size() > 1).map(Map.Entry::getKey).collect(Collectors.toList());

        //设置iccid重复提示
        iccidVerifyResults.stream().filter(v -> repeatIccIdList.contains(v.getIccid())).forEach(v -> {
            v.setExist(false);
            v.setMsg("iccid重复");
        });

        int failCount = (int) iccidVerifyResults.stream().filter(re -> !re.isExist()).count();
        log.info("failCount{}", failCount);

        String fileName = "";
        //如果失败记录，生成excel并上传
        ExcelSheetDTO excelSheetDTO = FileUtil.convertVerityResultToExcelDTO(iccidVerifyResults);
        //Response<FileRecordDTO> recordDTOResponse = fileService.createAndUploadExcel(fileId, "车卡信息校验结果.xlsx", excelSheetDTO);
		MultipartFile multipartFile;
		try {
			multipartFile = FileUtil.bytesToMultipartFile(FileUtil.createExcelBytes(excelSheetDTO, "车卡解绑信息校验结果.xlsx"), "车卡解绑信息校验结果.xlsx");
		} catch (Exception e) {
			log.error("生成校验文件失败", e);
			return Response.createError("生成校验文件失败", ResponseCode.INVALID_DATA.getCode());
		}

		FileUploadDTO dto = new FileUploadDTO();
		dto.setFile(multipartFile);
		Response<FileRecordDTO>  recordResponse = fileSystemClient.uploadFile(dto);
		if (!recordResponse.isSuccess()) {
			return Response.createError(recordResponse.getMsg(), recordResponse.getCode());
		}
		String fileleKey = recordResponse.getData().getUuid();

        return Response.createSuccess(new EMVinCardResponseDTO(iccidVerifyResults.size(), failCount, fileleKey, fileName));
    }
}
