package com.cusc.nirvana.user.rnr.enterprise.service.impl;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.rnr.enterprise.dto.EMVinCardResponseDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.ExcelSheetDTO;
import com.cusc.nirvana.user.rnr.enterprise.dto.VehicleRnrVinCardInfoDTO;
import com.cusc.nirvana.user.rnr.enterprise.service.IVehicleRnrUnboundService;
import com.cusc.nirvana.user.rnr.enterprise.util.FileUtil;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.client.VehicleUnboundClient;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrCompanyInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgVehicleCompanyClient;
import com.cusc.nirvana.user.rnr.mg.constants.*;
import com.cusc.nirvana.user.rnr.mg.dto.*;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 车企实名解绑实现类
 *
 * @author yubo
 * @since 2022-04-24 09:41
 */
@Service
@Slf4j
public class VehicleRnrUnboundServiceImpl implements IVehicleRnrUnboundService {

	@Resource
	private FileSystemClient fileSystemClient;

    @Autowired
    VehicleUnboundClient vehicleUnboundClient;

    @Autowired
    MgVehicleCompanyClient vehicleCompanyClient;

    @Autowired
    private MgRnrCompanyInfoClient companyInfoClient;

    @Autowired
    private MgRnrInfoClient rnrInfoClient;

    /**
     * 车企实名解绑校验车卡信息
     *
     * @param vehicleRnrVinCardInfoDTO
     */
    @Override
    public Response<EMVinCardResponseDTO> verifyUnboundVinCards(VehicleRnrVinCardInfoDTO vehicleRnrVinCardInfoDTO) {
        //获取校验结果
        Response<VerifyVinCardResponseDTO> response = getVerifyResults(vehicleRnrVinCardInfoDTO);
        if (!response.isSuccess()) {
            return Response.createError(response.getMsg(), response.getCode());
        }

        //校验结果处理，计算失败次数
        List<VinCardVerityResult> iccidVerifyResults = response.getData().getIccidVerifyResults();
        int failCount = getFailedCount(iccidVerifyResults);

        //如果失败记录，生成excel并上传
        ExcelSheetDTO excelSheetDTO = FileUtil.convertVerityResultToExcelDTO(iccidVerifyResults);

		MultipartFile multipartFile;
		try {
			multipartFile = FileUtil.bytesToMultipartFile(FileUtil.createExcelBytes(excelSheetDTO, "车卡解绑信息校验结果.xlsx"), "车卡解绑信息校验结果.xlsx");
		} catch (Exception e) {
			log.error("生成校验文件失败", e);
			return Response.createError("生成校验文件失败", ResponseCode.INVALID_DATA.getCode());
		}

		FileUploadDTO dto = new FileUploadDTO();
		dto.setFile(multipartFile);
		Response<FileRecordDTO>  recordResponse = fileSystemClient.uploadFile(dto);
		if (!recordResponse.isSuccess()) {
            return Response.createError(recordResponse.getMsg(), recordResponse.getCode());
        }
        String fileId = recordResponse.getData().getUuid();

        return Response.createSuccess(new EMVinCardResponseDTO(iccidVerifyResults.size(), failCount, fileId));
    }

    /**
     * 车企解绑
     *
     * @param vehicleRnrVinCardInfoDTO
     * @return
     */
    @Override
    public Response unboundSubmit(VehicleRnrVinCardInfoDTO vehicleRnrVinCardInfoDTO) {
        //获取校验结果
        Response<VehicleUnboundVerifyRequestDTO> verifyCard = getVerifyCard(vehicleRnrVinCardInfoDTO);
        if (!verifyCard.isSuccess()) {
            return Response.createSuccess(verifyCard.getMsg(), verifyCard.getCode());
        }

        //获取车企实名业务ID
        MgVehicleCompanyDTO fpVehicleCompanyDTO = new MgVehicleCompanyDTO();
        fpVehicleCompanyDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<MgVehicleCompanyDTO> byTenantNo = vehicleCompanyClient.getVehicleCompany(fpVehicleCompanyDTO);
        if (!byTenantNo.isSuccess() || byTenantNo.getData() == null) {
            return Response.createError("没有对应的车企信息", ResponseCode.INVALID_DATA.getCode());
        }
        fpVehicleCompanyDTO = byTenantNo.getData();

        //获取车企信息
        MgRnrCompanyInfoDTO companyInfoDTO = new MgRnrCompanyInfoDTO();
        companyInfoDTO.setRnrId(fpVehicleCompanyDTO.getFirstRnrId());
        Response<MgRnrCompanyInfoDTO> byRnrid = companyInfoClient.getByRnrid(companyInfoDTO);
        if (!byRnrid.isSuccess()) {
            return Response.createError(byRnrid.getMsg(), ResponseCode.INVALID_DATA.getCode());
        }
        if (byRnrid.getData() == null) {
            return Response.createError("没有对应的车企实名信息", ResponseCode.INVALID_DATA.getCode());
        }
        companyInfoDTO = byRnrid.getData();

        //获取实名信息
        MgRnrInfoDTO mgRnrInfoDTO = new MgRnrInfoDTO();
        mgRnrInfoDTO.setUuid(fpVehicleCompanyDTO.getFirstRnrId());
        Response<MgRnrInfoDTO> byUuid = rnrInfoClient.getByUuid(mgRnrInfoDTO);
        if (!byUuid.isSuccess() || byUuid.getData() == null) {
            return Response.createError("企业实名信息获取失败", ResponseCode.INVALID_DATA.getCode());
        }
        mgRnrInfoDTO = byUuid.getData();

        //数据转换
        VehicleUnbindDTO vehicleUnbindDTO = new VehicleUnbindDTO();
        vehicleUnbindDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        vehicleUnbindDTO.setIsTrust(0);
        vehicleUnbindDTO.setIsSecondHandCar(0);
        vehicleUnbindDTO.setInfo(getVehicleRnrInfo(mgRnrInfoDTO));
        vehicleUnbindDTO.setCompanyInfo(createCompanyInfoDTO(companyInfoDTO, vehicleUnbindDTO.getInfo().getUuid()));
        mgRnrInfoDTO.setRnrCompanyId(companyInfoDTO.getUuid());
        vehicleUnbindDTO.setOrder(getVehicleOrderInfo(vehicleUnbindDTO.getInfo(), companyInfoDTO));
        vehicleUnbindDTO.setCardList(getVehicleCardInfo(vehicleUnbindDTO.getInfo()
                , vehicleUnbindDTO.getOrder().getUuid(), verifyCard.getData().getCardDTOS()));
//        vehicleUnbindDTO.setRnrFileList(getVehicleFiles(vehicleUnbindDTO.getInfo(), byTenantNo.getData()));
        return vehicleUnboundClient.batchUnbound(vehicleUnbindDTO);
    }

    //------------- 私有方法区 -------------

	/**
	 * 公司信息
	 * @param mgRnrCompanyInfoDTO
	 * @param rnrId
	 * @return
	 */
    private MgRnrCompanyInfoDTO createCompanyInfoDTO(MgRnrCompanyInfoDTO mgRnrCompanyInfoDTO, String rnrId) {
        mgRnrCompanyInfoDTO.setUuid(CuscStringUtils.generateUuid());
        mgRnrCompanyInfoDTO.setRnrId(rnrId);
        mgRnrCompanyInfoDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        mgRnrCompanyInfoDTO.setCreator(UserSubjectUtil.getUserId());
        mgRnrCompanyInfoDTO.setOperator(UserSubjectUtil.getUserId());
        mgRnrCompanyInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
        //是否是车企
        mgRnrCompanyInfoDTO.setIndustryType("1");
        return mgRnrCompanyInfoDTO;
    }

    //获取文件信息
//    private List<MgRnrFileDTO> getVehicleFiles(MgRnrInfoDTO info, MgVehicleCompanyDTO data) {
//        List<MgRnrFileDTO> mgRnrFileDTOS = new ArrayList<>();
//
//        String rnrId = info.getUuid();
//        //mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, data.getCorporationCertFront(), RnrFileType.IDENTITY_CARD_BACK.getCode()));
//        //mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, data.getCorporationCertBack(), RnrFileType.IDENTITY_CARD_FRONT.getCode()));
//        //mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, data.getCompanyCertAuth(), RnrFileType.ENTERPRISE_AUTH_FILE.getCode()));
//        //mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, data.getLivenessVideoId(), RnrFileType.LIVENESS_VIDEO.getCode()));
//
//        return mgRnrFileDTOS;
//    }

	/**
	 * 获取车卡信息
	 * @param info
	 * @param orderId
	 * @param vinCardList
	 * @return
	 */
    private List<MgRnrCardInfoDTO> getVehicleCardInfo(MgRnrInfoDTO info, String orderId, List<VinCardDTO> vinCardList) {
        return vinCardList.stream().map(vc -> {
            MgRnrCardInfoDTO cardInfoDTO = new MgRnrCardInfoDTO();
            cardInfoDTO.setUuid(CuscStringUtils.generateUuid());
            cardInfoDTO.setIotId(vc.getVin());
            cardInfoDTO.setIccid(vc.getIccid());
            cardInfoDTO.setRnrId(info.getUuid());
            cardInfoDTO.setOperator(UserSubjectUtil.getUserId());
            cardInfoDTO.setSerial_number(info.getSerial_number());
            cardInfoDTO.setCreator(UserSubjectUtil.getUserId());
            cardInfoDTO.setOrderId(orderId);
            cardInfoDTO.setTenantNo(UserSubjectUtil.getTenantNo());
            cardInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
            return cardInfoDTO;
        }).collect(Collectors.toList());
    }

	/**
	 * 获取车企责订单信息
	 * @param mgRnrInfoDTO
	 * @param companyDTO
	 * @return
	 */
    private RnrOrderDTO getVehicleOrderInfo(MgRnrInfoDTO mgRnrInfoDTO, MgRnrCompanyInfoDTO companyDTO) {
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setOrderType(RnrOrderType.COMPANY_CORPORATION_UNBIND.getCode());
        rnrOrderDTO.setRnrId(mgRnrInfoDTO.getUuid());
        rnrOrderDTO.setUuid(CuscStringUtils.generateUuid());
        rnrOrderDTO.setTenantNo(mgRnrInfoDTO.getTenantNo());
        rnrOrderDTO.setOperator(mgRnrInfoDTO.getUserId());
        rnrOrderDTO.setCreator(mgRnrInfoDTO.getUserId());
        rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.PORTAL.getCode());
        rnrOrderDTO.setSerialNumber(mgRnrInfoDTO.getSerial_number());
        rnrOrderDTO.setIsBatchOrder(0);
        rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.TO_EXAMINE.getCode());
        rnrOrderDTO.setAutoRnr(true);
        rnrOrderDTO.setAuditType(RnrOrderAuditTypeEnum.AUTO.getCode());
        rnrOrderDTO.setSendWorkOrder(false);
        rnrOrderDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
		rnrOrderDTO.setOrgId(mgRnrInfoDTO.getOrgId());
        return rnrOrderDTO;
    }

	/**
	 * 获取车企责任人信息
	 * @param mgRnrInfoDTO
	 * @return
	 */
	private MgRnrInfoDTO getVehicleRnrInfo(MgRnrInfoDTO mgRnrInfoDTO) {
        mgRnrInfoDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        mgRnrInfoDTO.setSerial_number(CuscStringUtils.generateUuid());
        mgRnrInfoDTO.setOperator(UserSubjectUtil.getUserId());
        mgRnrInfoDTO.setCreator(UserSubjectUtil.getUserId());
        mgRnrInfoDTO.setUuid(CuscStringUtils.generateUuid());
        mgRnrInfoDTO.setUserId(UserSubjectUtil.getUserId());
        mgRnrInfoDTO.setIsCompany(1);
        mgRnrInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
        return mgRnrInfoDTO;
    }

    /**
     * 获取错误数量
     *
     * @param iccidVerifyResults
     * @return
     */
    private int getFailedCount(List<VinCardVerityResult> iccidVerifyResults) {
        return (int) iccidVerifyResults.stream().filter(re -> !re.isExist()).count();
    }

    /**
     * 获取解绑结果
     *
     * @param vehicleRnrVinCardInfoDTO
     * @return
     */
    private Response<VerifyVinCardResponseDTO> getVerifyResults(VehicleRnrVinCardInfoDTO vehicleRnrVinCardInfoDTO) {
        Response<VehicleUnboundVerifyRequestDTO> verifyCard = getVerifyCard(vehicleRnrVinCardInfoDTO);
        if (!verifyCard.isSuccess()) {
            return Response.createSuccess(verifyCard.getMsg(), verifyCard.getCode());
        }
//        return mockResut(verifyCard.getData());
        return vehicleUnboundClient.verifyUnboundVinCardBatch(verifyCard.getData());
    }

//    private Response<VerifyVinCardResponseDTO> mockResut(VehicleUnboundVerifyRequestDTO data) {
//        List<VinCardVerityResult> collect = data.getCardDTOS().stream().map(card -> {
//            VinCardVerityResult result = new VinCardVerityResult();
//            result.setVin(card.getVin());
//            result.setIccid(card.getIccid());
//            boolean b = new Random().nextBoolean();
//            result.setExist(b);
//            result.setMsg(b ? "" : "Error....");
//            return result;
//        }).collect(Collectors.toList());
//        VerifyVinCardResponseDTO verifyVinCardResponseDTO = new VerifyVinCardResponseDTO();
//        verifyVinCardResponseDTO.setIccidVerifyResults(collect);
//        return Response.createSuccess(verifyVinCardResponseDTO);
//    }

    /**
     * 获取检验结果
     *
     * @return
     */
    private Response<VehicleUnboundVerifyRequestDTO> getVerifyCard(VehicleRnrVinCardInfoDTO vehicleRnrVinCardInfoDTO) {
        if (StringUtils.isBlank(vehicleRnrVinCardInfoDTO.getFileId())) {
            return Response.createError("文件ID不能为空", ResponseCode.INVALID_DATA.getCode());
        }

        List<VinCardDTO> cardList = null;
        try {
            //cardDTOList = fileService.downloadAndToVinCardList(vehicleRnrVinCardInfoDTO.getFileId());
			FileDownloadDTO dto = new com.cusc.nirvana.user.rnr.enterprise.dto.FileDownloadDTO();
			dto.setUuid(vehicleRnrVinCardInfoDTO.getFileId());
			String base64 = fileSystemClient.getBase64(dto).getData().getBase64();
			byte[] decode = Base64.getDecoder().decode(base64);
			ByteArrayInputStream inputStream = new ByteArrayInputStream(decode);
			cardList = FileUtil.convertExcelToVinCardList(inputStream, vehicleRnrVinCardInfoDTO.getFileId());
		} catch (Exception e) {
            log.error("读取车卡列表失败", e);
            return Response.createError("读取车卡列表失败", ResponseCode.INVALID_DATA.getCode());
        }

        if (CollectionUtils.isEmpty(cardList)) {
            return Response.createError("未读取到车卡信息", ResponseCode.INVALID_DATA.getCode());
        }

        VehicleUnboundVerifyRequestDTO verifyRequestDTO = new VehicleUnboundVerifyRequestDTO();
        verifyRequestDTO.setCardDTOS(cardList);
        verifyRequestDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        verifyRequestDTO.setUserId(UserSubjectUtil.getUserId());

        return Response.createSuccess(verifyRequestDTO);
    }
}
