package com.cusc.nirvana.user.rnr.enterprise.util;

import com.alibaba.fastjson.JSON;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.enterprise.dto.ExcelSheetDTO;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.fp.dto.VerifyExcelStatusEnum;
import com.cusc.nirvana.user.rnr.fp.dto.VinCardDTO;
import com.cusc.nirvana.user.rnr.fp.dto.VinCardVerityResult;
import com.cusc.nirvana.user.rnr.mg.constants.CertTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrBizzTypeEnum;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrFileDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.http.MediaType;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * 文件处理类
 *
 * @author yubo
 * @since 2022-04-19 10:12
 */
public class FileUtil {

    /**
     * 车企和企业的excel转成车卡list
     *
     * @param inputStream
     * @return
     */
    public static List<VinCardDTO> convertExcelToVinCardList(InputStream inputStream, String fileName) throws IOException {
        return convertExcelToVinCardList(inputStream, isExcelXSSFFormat(fileName), 1);
    }

    public static List<VinCardVerityResult> convertExcelToVinCardLists(InputStream inputStream, String fileName) throws IOException {
        return convertExcelToVinCardLists(inputStream, isExcelXSSFFormat(fileName), 1);
    }

    //创建文件信息
    public static MgRnrFileDTO newFileDTO(String rnrId, String pic, Integer fileType, int rnrBizzType) {
        return newFileDTO(rnrId, pic, fileType, "0", rnrBizzType);
    }

    //创建文件信息
    public static MgRnrFileDTO newFileDTO(String rnrId, String pic, Integer fileType, String liasionId,int rnrBizzType) {
        MgRnrFileDTO mgRnrFileDTO = new MgRnrFileDTO();
        mgRnrFileDTO.setUuid(CuscStringUtils.generateUuid());
        mgRnrFileDTO.setRnrId(rnrId);
        mgRnrFileDTO.setFileType(fileType);
        mgRnrFileDTO.setFileSystemId(pic);
        mgRnrFileDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        mgRnrFileDTO.setLiaisonId(liasionId);
        mgRnrFileDTO.setCreator(UserSubjectUtil.getUserId());
        mgRnrFileDTO.setOperator(UserSubjectUtil.getUserId());
        mgRnrFileDTO.setRnrBizzType(rnrBizzType);
        return mgRnrFileDTO;
    }

    /**
     * 车企和企业的excel转成车卡list
     *
     * @param usingXSSF 是否使用xssf
     * @param skipLines 跳过的行数，用于跳过列名行
     * @return
     */
    public static List<VinCardDTO> convertExcelToVinCardList(InputStream inputStream, boolean usingXSSF, int skipLines) throws IOException {
        try (Workbook wb = usingXSSF ? new XSSFWorkbook(inputStream) : new HSSFWorkbook(inputStream)) {
            Sheet sheet = wb.getSheetAt(0);
            Iterator<Row> iterator = sheet.iterator();

            List<VinCardDTO> vinCardDTOS = new ArrayList<>();

            int index = 0;
            while (iterator.hasNext()) {
                Row next = iterator.next();
                if (index < skipLines) {
                    index++;
                    continue;
                }
                //如果读取到的第一列的数据为空，则跳过
                String cellValue = getCellValue(next.getCell(0));
                if(StringUtils.isBlank(cellValue)){
                    continue;
                }
                vinCardDTOS.add(new VinCardDTO(cellValue, getCellValue(next.getCell(1))));
            }

            return vinCardDTOS;
        }
    }

    /**
     * 获取文件类型
     *
     * @param certType 证书类型
     * @param index    索引
     */
    public static Integer getFileType(String certType, int index) {
        CertTypeEnum typeEnum = CertTypeEnum.getEnumByCode(certType);

        if (index < 0 || typeEnum == null || typeEnum.getFileTypes() == null || typeEnum.getFileTypes().length < 0) {
            throw new CuscUserException(ResponseCode.INVALID_DATA.getCode(), "证件类型【" + certType + "】错误");
        }

        //不能超过固定的文件数量
        if (typeEnum.isFixedSize() && index >= typeEnum.getFileTypes().length) {
            throw new CuscUserException(ResponseCode.INVALID_DATA.getCode(),
                    "证件【" + typeEnum.getName() + "】上传文件数超过指定数量【" + typeEnum.getFileTypes().length + "】");
        }

        if (index >= typeEnum.getFileTypes().length) {
            index = typeEnum.getFileTypes().length - 1;
        }
        return typeEnum.getFileTypes()[index].getCode();
    }

    public static List<VinCardVerityResult> convertExcelToVinCardLists(InputStream inputStream, boolean usingXSSF, int skipLines) throws IOException {
        try (Workbook wb = usingXSSF ? new XSSFWorkbook(inputStream) : new HSSFWorkbook(inputStream)) {
            Sheet sheet = wb.getSheetAt(0);
            Iterator<Row> iterator = sheet.iterator();

            List<VinCardVerityResult> vinCardDTOS = new ArrayList<>();

            int index = 0;
            while (iterator.hasNext()) {
                Row next = iterator.next();
                if (index < skipLines) {
                    index++;
                    continue;
                }
                //如果读取到的第一列的数据为空，表示已经读到最后，跳出
                String cellValue = getCellValue(next.getCell(0));
                if(StringUtils.isBlank(cellValue)){
                    continue;
                }
                vinCardDTOS.add(new VinCardVerityResult(cellValue, getCellValue(next.getCell(1)),
                        StringUtils.equals(getCellValue(next.getCell(2)), VerifyExcelStatusEnum.SUCCESS.getCode())
                        , getCellValue(next.getCell(3))));
            }

            return vinCardDTOS;
        }
    }

    /**
     * 获取单元格值
     *
     * @param cell
     * @return
     */
    private static String getCellValue(Cell cell) {
        if (cell == null) {
            return "";
        }
        CellType cellType = cell.getCellType();
        switch (cellType) {
            case STRING:
                return cell.getStringCellValue();
            case NUMERIC:
                return String.valueOf((int) cell.getNumericCellValue());
            case BOOLEAN:
                return String.valueOf(cell.getBooleanCellValue());
            default:
                return "";
        }
    }

    /**
     * 校验结果转成excel数据
     */
    public static ExcelSheetDTO convertVerityResultToExcelDTO(List<VinCardVerityResult> verityResults) {
        ExcelSheetDTO excelSheetDTO = new ExcelSheetDTO();
        excelSheetDTO.setSheetName("验证结果列表");
        excelSheetDTO.setSchema(new String[]{"VIN", "ICCID", "状态", "错误信息"});

        if (!verityResults.isEmpty()) {
            for (VinCardVerityResult verityResult : verityResults) {
                excelSheetDTO.getData().add(new String[]{verityResult.getVin(), verityResult.getIccid()
                        , verityResult.isExist() ? VerifyExcelStatusEnum.SUCCESS.getCode() : VerifyExcelStatusEnum.FAILED.getCode()
                        , verityResult.getMsg()});
            }
        }

        return excelSheetDTO;
    }

    /**
     * 通过文件名判断excel格式
     *
     * @param fileName
     * @return
     */
    public static boolean isExcelXSSFFormat(String fileName) {
        return StringUtils.isNotBlank(fileName) && fileName.toLowerCase().endsWith(".xlsx");
    }

    public static MultipartFile bytesToMultipartFile(byte[] bytes, String fileName) {
        FileItem fileItem = new DiskFileItemFactory().createItem("file",
                MediaType.MULTIPART_FORM_DATA_VALUE, true, fileName);

        try (InputStream input = new ByteArrayInputStream(bytes); OutputStream os = fileItem.getOutputStream()) {
            IOUtils.copy(input, os);
        } catch (Exception e) {
            throw new IllegalArgumentException("Invalid file: " + e, e);
        }

        return new CommonsMultipartFile(fileItem);
    }

    public static byte[] createExcelBytes(ExcelSheetDTO excelSheetDTO, String fileName) throws IOException {
        return createExcelBytes(excelSheetDTO, isExcelXSSFFormat(fileName));
    }

    private static byte[] createExcelBytes(ExcelSheetDTO excelSheetDTO, boolean excelXSSFFormat) throws IOException {
        Workbook workbook = excelXSSFFormat ? new XSSFWorkbook() : new HSSFWorkbook();
        Sheet sheet = workbook.createSheet(excelSheetDTO.getSheetName());

        //写入列名行
        int rowIndex = 0, colIndex = 0;
        Row row = sheet.createRow(rowIndex++);
        for (String name : excelSheetDTO.getSchema()) {
            row.createCell(colIndex++).setCellValue(name);
        }

        //写入信息
        for (String[] colData : excelSheetDTO.getData()) {
            colIndex = 0;
            row = sheet.createRow(rowIndex++);
            for (String cellValue : colData) {
                row.createCell(colIndex++).setCellValue(cellValue);
            }
        }

        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream(1024);
        workbook.write(byteArrayOutputStream);
        return byteArrayOutputStream.toByteArray();
    }
}
